@@ -43,66 +43,65 @@ class Region;
 
 namespace polly {
 
-/// @brief Type to hold region delimitors (entry & exit block).
+/// Type to hold region delimiters (entry & exit block).
 using BBPair = std::pair<BasicBlock *, BasicBlock *>;
 
-/// @brief Return the region delimitors (entry & exit block) of @p R.
+/// Return the region delimiters (entry & exit block) of @p R.
 BBPair getBBPairForRegion(const Region *R);
 
-/// @brief Set the begin and end source location for the region limited by @p P.
+/// Set the begin and end source location for the region limited by @p P.
 void getDebugLocations(const BBPair &P, DebugLoc &Begin, DebugLoc &End);
 
 class RejectLog;
-/// @brief Emit optimization remarks about the rejected regions to the user.
+/// Emit optimization remarks about the rejected regions to the user.
 ///
 /// This emits the content of the reject log as optimization remarks.
 /// Remember to at least track failures (-polly-detect-track-failures).
-/// @param P The region delimitors (entry & exit) we emit remarks for.
+/// @param P The region delimiters (entry & exit) we emit remarks for.
 /// @param Log The error log containing all messages being emitted as remark.
 void emitRejectionRemarks(const BBPair &P, const RejectLog &Log);
 
 // Discriminator for LLVM-style RTTI (dyn_cast<> et al.)
-enum RejectReasonKind {
+enum class RejectReasonKind {
   // CFG Category
-  rrkCFG,
-  rrkInvalidTerminator,
-  rrkCondition,
-  rrkLastCFG,
-  rrkIrreducibleRegion,
+  CFG,
+  InvalidTerminator,
+  IrreducibleRegion,
+  LastCFG,
 
   // Non-Affinity
-  rrkAffFunc,
-  rrkUndefCond,
-  rrkInvalidCond,
-  rrkUndefOperand,
-  rrkNonAffBranch,
-  rrkNoBasePtr,
-  rrkUndefBasePtr,
-  rrkVariantBasePtr,
-  rrkNonAffineAccess,
-  rrkDifferentElementSize,
-  rrkLastAffFunc,
+  AffFunc,
+  UndefCond,
+  InvalidCond,
+  UndefOperand,
+  NonAffBranch,
+  NoBasePtr,
+  UndefBasePtr,
+  VariantBasePtr,
+  NonAffineAccess,
+  DifferentElementSize,
+  LastAffFunc,
 
-  rrkLoopBound,
-  rrkLoopOverlapWithNonAffineSubRegion,
+  LoopBound,
+  LoopHasNoExit,
 
-  rrkFuncCall,
-  rrkNonSimpleMemoryAccess,
+  FuncCall,
+  NonSimpleMemoryAccess,
 
-  rrkAlias,
+  Alias,
 
   // Other
-  rrkOther,
-  rrkIntToPtr,
-  rrkAlloca,
-  rrkUnknownInst,
-  rrkEntry,
-  rrkUnprofitable,
-  rrkLastOther
+  Other,
+  IntToPtr,
+  Alloca,
+  UnknownInst,
+  Entry,
+  Unprofitable,
+  LastOther
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Base class of all reject reasons found during Scop detection.
+/// Base class of all reject reasons found during Scop detection.
 ///
 /// Subclasses of RejectReason should provide means to capture enough
 /// diagnostic information to help clients figure out what and where something
@@ -118,16 +117,16 @@ protected:
 public:
   RejectReasonKind getKind() const { return Kind; }
 
-  RejectReason(RejectReasonKind K) : Kind(K) {}
+  RejectReason(RejectReasonKind K);
 
   virtual ~RejectReason() {}
 
-  /// @brief Generate a reasonable diagnostic message describing this error.
+  /// Generate a reasonable diagnostic message describing this error.
   ///
   /// @return A debug message representing this error.
   virtual std::string getMessage() const = 0;
 
-  /// @brief Generate a message for the end-user describing this error.
+  /// Generate a message for the end-user describing this error.
   ///
   /// The message provided has to be suitable for the end-user. So it should
   /// not reference any LLVM internal data structures or terminology.
@@ -137,7 +136,7 @@ public:
   /// @return A short message representing this error.
   virtual std::string getEndUserMessage() const { return "Unspecified error."; }
 
-  /// @brief Get the source location of this error.
+  /// Get the source location of this error.
   ///
   /// @return The debug location for this error.
   virtual const llvm::DebugLoc &getDebugLoc() const;
@@ -145,7 +144,7 @@ public:
 
 typedef std::shared_ptr<RejectReason> RejectReasonPtr;
 
-/// @brief Stores all errors that ocurred during the detection.
+/// Stores all errors that ocurred during the detection.
 class RejectLog {
   Region *R;
   llvm::SmallVector<RejectReasonPtr, 1> ErrorReports;
@@ -159,7 +158,7 @@ public:
   iterator end() const { return ErrorReports.end(); }
   size_t size() const { return ErrorReports.size(); }
 
-  /// @brief Returns true, if we store at least one error.
+  /// Returns true, if we store at least one error.
   ///
   /// @return true, if we store at least one error.
   bool hasErrors() const { return size() > 0; }
@@ -171,7 +170,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Base class for CFG related reject reasons.
+/// Base class for CFG related reject reasons.
 ///
 /// Scop candidates that violate structural restrictions can be grouped under
 /// this reject reason class.
@@ -187,13 +186,13 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures bad terminator within a Scop candidate.
+/// Captures bad terminator within a Scop candidate.
 class ReportInvalidTerminator : public ReportCFG {
   BasicBlock *BB;
 
 public:
   ReportInvalidTerminator(BasicBlock *BB)
-      : ReportCFG(rrkInvalidTerminator), BB(BB) {}
+      : ReportCFG(RejectReasonKind::InvalidTerminator), BB(BB) {}
 
   /// @name LLVM-RTTI interface
   //@{
@@ -208,14 +207,14 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures irreducible regions in CFG.
+/// Captures irreducible regions in CFG.
 class ReportIrreducibleRegion : public ReportCFG {
   Region *R;
   DebugLoc DbgLoc;
 
 public:
   ReportIrreducibleRegion(Region *R, DebugLoc DbgLoc)
-      : ReportCFG(rrkIrreducibleRegion), R(R), DbgLoc(DbgLoc) {}
+      : ReportCFG(RejectReasonKind::IrreducibleRegion), R(R), DbgLoc(DbgLoc) {}
 
   /// @name LLVM-RTTI interface
   //@{
@@ -231,7 +230,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Base class for non-affine reject reasons.
+/// Base class for non-affine reject reasons.
 ///
 /// Scop candidates that violate restrictions to affinity are reported under
 /// this class.
@@ -258,7 +257,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures a condition that is based on an 'undef' value.
+/// Captures a condition that is based on an 'undef' value.
 class ReportUndefCond : public ReportAffFunc {
   //===--------------------------------------------------------------------===//
 
@@ -267,7 +266,7 @@ class ReportUndefCond : public ReportAffFunc {
 
 public:
   ReportUndefCond(const Instruction *Inst, BasicBlock *BB)
-      : ReportAffFunc(rrkUndefCond, Inst), BB(BB) {}
+      : ReportAffFunc(RejectReasonKind::UndefCond, Inst), BB(BB) {}
 
   /// @name LLVM-RTTI interface
   //@{
@@ -281,7 +280,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures an invalid condition
+/// Captures an invalid condition
 ///
 /// Conditions have to be either constants or icmp instructions.
 class ReportInvalidCond : public ReportAffFunc {
@@ -292,7 +291,7 @@ class ReportInvalidCond : public ReportAffFunc {
 
 public:
   ReportInvalidCond(const Instruction *Inst, BasicBlock *BB)
-      : ReportAffFunc(rrkInvalidCond, Inst), BB(BB) {}
+      : ReportAffFunc(RejectReasonKind::InvalidCond, Inst), BB(BB) {}
 
   /// @name LLVM-RTTI interface
   //@{
@@ -306,7 +305,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures an undefined operand.
+/// Captures an undefined operand.
 class ReportUndefOperand : public ReportAffFunc {
   //===--------------------------------------------------------------------===//
 
@@ -315,7 +314,7 @@ class ReportUndefOperand : public ReportAffFunc {
 
 public:
   ReportUndefOperand(BasicBlock *BB, const Instruction *Inst)
-      : ReportAffFunc(rrkUndefOperand, Inst), BB(BB) {}
+      : ReportAffFunc(RejectReasonKind::UndefOperand, Inst), BB(BB) {}
 
   /// @name LLVM-RTTI interface
   //@{
@@ -329,14 +328,14 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures a non-affine branch.
+/// Captures a non-affine branch.
 class ReportNonAffBranch : public ReportAffFunc {
   //===--------------------------------------------------------------------===//
 
   // The BasicBlock we found the non-affine branch in.
   BasicBlock *BB;
 
-  /// @brief LHS & RHS of the failed condition.
+  /// LHS & RHS of the failed condition.
   //@{
   const SCEV *LHS;
   const SCEV *RHS;
@@ -345,7 +344,8 @@ class ReportNonAffBranch : public ReportAffFunc {
 public:
   ReportNonAffBranch(BasicBlock *BB, const SCEV *LHS, const SCEV *RHS,
                      const Instruction *Inst)
-      : ReportAffFunc(rrkNonAffBranch, Inst), BB(BB), LHS(LHS), RHS(RHS) {}
+      : ReportAffFunc(RejectReasonKind::NonAffBranch, Inst), BB(BB), LHS(LHS),
+        RHS(RHS) {}
 
   const SCEV *lhs() { return LHS; }
   const SCEV *rhs() { return RHS; }
@@ -362,12 +362,12 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures a missing base pointer.
+/// Captures a missing base pointer.
 class ReportNoBasePtr : public ReportAffFunc {
   //===--------------------------------------------------------------------===//
 public:
   ReportNoBasePtr(const Instruction *Inst)
-      : ReportAffFunc(rrkNoBasePtr, Inst) {}
+      : ReportAffFunc(RejectReasonKind::NoBasePtr, Inst) {}
 
   /// @name LLVM-RTTI interface
   //@{
@@ -381,12 +381,12 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures an undefined base pointer.
+/// Captures an undefined base pointer.
 class ReportUndefBasePtr : public ReportAffFunc {
   //===--------------------------------------------------------------------===//
 public:
   ReportUndefBasePtr(const Instruction *Inst)
-      : ReportAffFunc(rrkUndefBasePtr, Inst) {}
+      : ReportAffFunc(RejectReasonKind::UndefBasePtr, Inst) {}
 
   /// @name LLVM-RTTI interface
   //@{
@@ -400,7 +400,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures a base pointer that is not invariant in the region.
+/// Captures a base pointer that is not invariant in the region.
 class ReportVariantBasePtr : public ReportAffFunc {
   //===--------------------------------------------------------------------===//
 
@@ -409,7 +409,8 @@ class ReportVariantBasePtr : public ReportAffFunc {
 
 public:
   ReportVariantBasePtr(Value *BaseValue, const Instruction *Inst)
-      : ReportAffFunc(rrkVariantBasePtr, Inst), BaseValue(BaseValue) {}
+      : ReportAffFunc(RejectReasonKind::VariantBasePtr, Inst),
+        BaseValue(BaseValue) {}
 
   /// @name LLVM-RTTI interface
   //@{
@@ -424,7 +425,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures a non-affine access function.
+/// Captures a non-affine access function.
 class ReportNonAffineAccess : public ReportAffFunc {
   //===--------------------------------------------------------------------===//
 
@@ -437,8 +438,8 @@ class ReportNonAffineAccess : public ReportAffFunc {
 public:
   ReportNonAffineAccess(const SCEV *AccessFunction, const Instruction *Inst,
                         const Value *V)
-      : ReportAffFunc(rrkNonAffineAccess, Inst), AccessFunction(AccessFunction),
-        BaseValue(V) {}
+      : ReportAffFunc(RejectReasonKind::NonAffineAccess, Inst),
+        AccessFunction(AccessFunction), BaseValue(V) {}
 
   const SCEV *get() { return AccessFunction; }
 
@@ -455,7 +456,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Report array accesses with differing element size.
+/// Report array accesses with differing element size.
 class ReportDifferentArrayElementSize : public ReportAffFunc {
   //===--------------------------------------------------------------------===//
 
@@ -464,7 +465,8 @@ class ReportDifferentArrayElementSize : public ReportAffFunc {
 
 public:
   ReportDifferentArrayElementSize(const Instruction *Inst, const Value *V)
-      : ReportAffFunc(rrkDifferentElementSize, Inst), BaseValue(V) {}
+      : ReportAffFunc(RejectReasonKind::DifferentElementSize, Inst),
+        BaseValue(V) {}
 
   /// @name LLVM-RTTI interface
   //@{
@@ -479,7 +481,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures errors with non affine loop bounds.
+/// Captures errors with non affine loop bounds.
 class ReportLoopBound : public RejectReason {
   //===--------------------------------------------------------------------===//
 
@@ -511,22 +513,19 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures errors when loop overlap with nonaffine subregion.
-class ReportLoopOverlapWithNonAffineSubRegion : public RejectReason {
+/// Captures errors when loop has no exit.
+class ReportLoopHasNoExit : public RejectReason {
   //===--------------------------------------------------------------------===//
 
-  /// @brief If L and R are set then L and R overlap.
-
-  /// The loop contains stmt overlapping nonaffine subregion.
+  /// The loop that has no exit.
   Loop *L;
 
-  /// The nonaffine subregion that contains infinite loop.
-  Region *R;
-
   const DebugLoc Loc;
 
 public:
-  ReportLoopOverlapWithNonAffineSubRegion(Loop *L, Region *R);
+  ReportLoopHasNoExit(Loop *L)
+      : RejectReason(RejectReasonKind::LoopHasNoExit), L(L),
+        Loc(L->getStartLoc()) {}
 
   /// @name LLVM-RTTI interface
   //@{
@@ -542,7 +541,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures errors with non-side-effect-known function calls.
+/// Captures errors with non-side-effect-known function calls.
 class ReportFuncCall : public RejectReason {
   //===--------------------------------------------------------------------===//
 
@@ -566,14 +565,14 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures errors with aliasing.
+/// Captures errors with aliasing.
 class ReportAlias : public RejectReason {
   //===--------------------------------------------------------------------===//
 public:
   typedef std::vector<const llvm::Value *> PointerSnapshotTy;
 
 private:
-  /// @brief Format an invalid alias set.
+  /// Format an invalid alias set.
   ///
   //  @param Prefix A prefix string to put before the list of aliasing pointers.
   //  @param Suffix A suffix string to put after the list of aliasing pointers.
@@ -604,7 +603,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Base class for otherwise ungrouped reject reasons.
+/// Base class for otherwise ungrouped reject reasons.
 class ReportOther : public RejectReason {
   //===--------------------------------------------------------------------===//
 public:
@@ -622,7 +621,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures errors with bad IntToPtr instructions.
+/// Captures errors with bad IntToPtr instructions.
 class ReportIntToPtr : public ReportOther {
   //===--------------------------------------------------------------------===//
 
@@ -645,7 +644,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures errors with alloca instructions.
+/// Captures errors with alloca instructions.
 class ReportAlloca : public ReportOther {
   //===--------------------------------------------------------------------===//
   Instruction *Inst;
@@ -666,7 +665,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures errors with unknown instructions.
+/// Captures errors with unknown instructions.
 class ReportUnknownInst : public ReportOther {
   //===--------------------------------------------------------------------===//
   Instruction *Inst;
@@ -687,7 +686,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures errors with regions containing the function entry block.
+/// Captures errors with regions containing the function entry block.
 class ReportEntry : public ReportOther {
   //===--------------------------------------------------------------------===//
   BasicBlock *BB;
@@ -708,7 +707,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Report regions that seem not profitable to be optimized.
+/// Report regions that seem not profitable to be optimized.
 class ReportUnprofitable : public ReportOther {
   //===--------------------------------------------------------------------===//
   Region *R;
@@ -730,7 +729,7 @@ public:
 };
 
 //===----------------------------------------------------------------------===//
-/// @brief Captures errors with non-simple memory accesses.
+/// Captures errors with non-simple memory accesses.
 class ReportNonSimpleMemoryAccess : public ReportOther {
   //===--------------------------------------------------------------------===//
 
